/**************************************************************************************************
* File: MP3_Test.c
* File Type: C - Source Code File
* (C) mikroElektronika, 2010-2012
* Modified for FreeRTOS by:
      Stevan A. Milinkovic
      Union University, School of Computing
      Knez Mihailova 6, 11000 Belgrade, Serbia
      e-mail: smilinkovic@raf.edu.rs
* Revision History:
*  - Initial version
* Description:
*     This project demonstrates communication with VS1053B mp3 codec.
*     Program reads one mp3 file from MMC and sends it to VS1053B for decoding
*     and playing.
*     MMC and MP3_SCI share Hardware SPI module.
* Test configuration:
*      MCU:              STM32F107VC
                         http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
       Dev.Board:        EasyMx v7 for STM32  ARM 
                         http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
                      mx7
*       Modules:         MP3 VS1053B on-board module
*                     dec
*                        EasyTFT
                         
*       Oscillator:      HSE-PLL, 72.0000 MHz
*       SW:              mikroC PRO for ARM
*                        http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-arm/
* NOTES:
*     - Make sure that MMC card is properly formatted (to FAT16 or just FAT)
*       before testing it on this example.
*     - Make sure that MMC card contains appropriate mp3 file ( sound.mp3 ).
*     - Turn on SPI and audio switches SW13.1 to SW13.7 and turn on microSD card
*       switches SW14.3 and SW14.4
**************************************************************************************************/
#include <built_in.h>
#include "MP3_Test.h"
#include "FreeRTOS.h"
#include "task.h"

#define TaskDelay_ms(x)             vTaskDelay( x/portTICK_RATE_MS )

/**************************************************************************************************
* MMC Chip Select connection
**************************************************************************************************/
sbit Mmc_Chip_Select           at GPIOD_ODR.B3;
/**************************************************************************************************
* CODEC V1053E connections
**************************************************************************************************/
sbit MP3_CS                    at GPIOC_ODR.B8;
sbit MP3_RST                   at GPIOC_ODR.B7;
sbit BSYNC                     at GPIOC_ODR.B9;
sbit DREQ                      at GPIOC_IDR.B6;

/**************************************************************************************************
* Function MCU_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize MCU
* Input: Nothing
* Output: Nothing
**************************************************************************************************/

void InitMCU(){
  // MP3 config
  GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_7);    // RST
  GPIO_Digital_Input (&GPIOC_BASE, _GPIO_PINMASK_6);    // DREQ
  GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_8);    // CS
  GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_9);    // DCS
}

/**************************************************************************************************
* Function MC3_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize MP3 VS1053B codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/

void MP3_Init(void) {
  BSYNC = 1;
  MP3_CS = 1;

  // Hardware reset
  MP3_RST = 0;
  Delay_ms(10);
  MP3_RST = 1;

  while (DREQ == 0) taskYIELD();

  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0x2000);   // default 12 288 000 Hz

  // Maximum volume is 0x00 and total silence is 0xFE.
  volume_left  = 0; //0x3F;
  volume_right = 0; //0x3F;
  MP3_Set_Volume(volume_left, volume_right);
}

/**************************************************************************************************
* Function MP3_Start()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize SPI to communicate with MP3 codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Start(void) {
  MP3_CS            = 1;               // Deselect MP3_CS
  MP3_RST           = 1;               // Set MP3_RST pin

  BSYNC             = 0;               // Clear BSYNC
  BSYNC             = 1;               // Clear BSYNC

  // Initialize SPIC module
  SPI3_Init_Advanced(_SPI_FPCLK_DIV16, _SPI_MASTER  | _SPI_8_BIT |
                     _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                     &_GPIO_MODULE_SPI3_PC10_11_12);
  MP3_Init();
  TaskDelay_ms(500);
}

/**************************************************************************************************
* Function MP3_Test()
* -------------------------------------------------------------------------------------------------
* Overview: Function run the MP3 test
* Input: Nothing
* Output: test status: 0 - skiped; 1 - pass; 2 - fail
**************************************************************************************************/
void MP3_Test(char *test) {
  unsigned long i;

  // Reset error flag
  *test = 0;

 // Initializing MMC_FAT
  if (Mmc_Fat_Init() == 0) {
    if (Mmc_Fat_Assign(&mp3_filename, 0) ) {
      // File Assigned
      Mmc_Fat_Reset(&file_size);          // Call Reset before file reading,
                                          // procedure returns size of the file
      // Play audio...
      // send file blocks to MP3 SDI
    while (file_size > BUFFER_SIZE) {
        for (i=0; i<BUFFER_SIZE; i++) {
          Mmc_Fat_Read(mp3_buffer + i);
        }
        taskYIELD();
        vTaskSuspendAll();
        for (i=0; i<BUFFER_SIZE/BYTES_2_WRITE; i++) {
          MP3_SDI_Write_32(mp3_buffer + i*BYTES_2_WRITE);
        }
        xTaskResumeAll();
        taskYIELD();
        file_size -= BUFFER_SIZE;
    }
      // send the rest of the file to MP3 SDI
        for (i=0; i<file_size; i++) {
          Mmc_Fat_Read(mp3_buffer + i);
          taskYIELD();
        }
        for (i=0; i<file_size; i++) {
          MP3_SDI_Write(mp3_buffer[i]);
          taskYIELD();
        }
      // Finish
    }
    else {
      // File not assigned
      *test = 2;
    }
  }
  else {
    // MMC FAT not initialized
    *test = 2;
  }
}